/***************************************************************************
 *            PSMSContentDatabase.h
 *
 *  
 *  libpsms  - library of Promise SmartStor Media Server
 *
 *  Copyright (C) 2007 Promise Technology Inc.  
 ****************************************************************************/

#ifndef _PSMSCONTENTDATABASE_H
#define _PSMSCONTENTDATABASE_H

#include <sqlite3.h>
#include <string>
#include <map>
#include <list>
#include "Common/Common.h"
#include "ContentDirectory/ContentDatabase.h"

class CPSMSSelectResult
{
  public:
    std::string  GetValue(std::string p_sFieldName);
    std::map<std::string, std::string> m_FieldValues;
    std::map<std::string, std::string>::iterator m_FieldValuesIterator;  
};

class CPSMSContentDatabase
{
  public:
    static CPSMSContentDatabase* Shared();  
  
    CPSMSContentDatabase();
    ~CPSMSContentDatabase();
  
    std::string GetLibVersion();

    bool Init(bool* p_bIsNewDB);
  
    unsigned int Insert(std::string p_sStatement);
    bool Select(std::string p_sStatement);
  
    bool Eof();
    CPSMSSelectResult* GetResult();
    void Next();
  
    std::list<CPSMSSelectResult*> m_ResultList;
    std::list<CPSMSSelectResult*>::iterator m_ResultListIterator;
    unsigned int  m_nRowsReturned;  
  
    void ClearResult();
    void Lock();
    void Unlock();
  
    void BuildDB();
    int CheckSharedDirTotalSize();
    bool IsRebuilding() { return m_bIsRebuilding; };
    bool g_bExitApp;
  private:    
    void DbScanDir(std::string p_sDirectory, long long int p_nParentId);
    bool m_bIsRebuilding;
    static CPSMSContentDatabase* m_Instance;
    fuppesThreadMutex        m_Mutex;
    sqlite3*                 m_pDbHandle;  
    std::string              m_sDbFileName;    
    bool Open();
    void Close();
};

#endif /* _CONTENTDATABASE_H */
